#include "jserver.h"


xmlnode js_config(char *query)
{
    static xmlnode config = NULL;

    log_debug(ZONE,"config query %s",query);

    if(config == NULL)
    {
        config = xmlnode_file(query);
        if(config == NULL)
        {
            log_error("jserver","Configuration using %s failed",query);
            printf("Configuration using %s failed\n",query);
            return NULL;
        }
        return config;
    }

    if(query == NULL)
        return config;
    else
        return xmlnode_get_tag(config, query);
}


struct name_list
{
    char *name;
    struct name_list *next;
};

/* this is a special internal name resolver for the dns name(s) of this transport, to check if a name is local */
int js_config_name(command cmd, char *name)
{
    static struct name_list *names;
    struct name_list *n, *i;
    xmlnode cur;

    log_debug(ZONE,"name %d for %s",cmd,name);

    switch(cmd)
    {
        case C_SET: /* only intended to be run at config/startup time */
            if(js_config_name(C_CHECK,name) || name == NULL)
                return 0;
            n = malloc(sizeof(struct name_list));
            n->name = strdup(name);
            n->next = names;
            names = n;
            break;
        case C_CHECK:
            for(i = names; i != NULL; i = i->next)
                if(j_strcmp(name,i->name) == 0)
                    return 1;
            break;
        case C_INIT:
            for(cur = xmlnode_get_firstchild(js_config("names")); cur != NULL; cur = xmlnode_get_nextsibling(cur))
                if(xmlnode_get_type(cur) == NTYPE_TAG)
                    js_config_name(C_SET,xmlnode_get_data(cur));
            break;
    }
    return 0;
}

