#include "svc_http.h"

/* HTTP Service: /public/*
 * 
 * 
 */

ppdb svc_http_public_presence_cache;

int svc_http_public_presence(hrq rq, xmlnode x)
{
    xmlnode pres, asked, cur;
    char *user, *askstr, *idstr, *ext, *res;
    jid id, from;

    if(x == NULL) return 0;
    user = xmlnode_get_attrib(x,"FILE");
    if(user == NULL || strlen(user) == 0) return 0;
    ext = xmlnode_get_attrib(x,"EXT");
    if(ext == NULL || strlen(ext) == 0) return 0;

    /* the user we're curious about */
    id = jid_new(rq->p,xmlnode_get_attrib(rq->x,"Host"));
    jid_set(id,user,JID_USER);
    idstr = jid_full(id);
    idstr = strstr(idstr,":");
    ++idstr; /* just the user@host */

    asked = js_xdb_get(NULL,"jabber:http:asked");
    if(asked == NULL)
        asked = xmlnode_new_tag("asked");
    askstr = xmlnode_get_data(asked);
    if(askstr == NULL || strstr(askstr,idstr) == NULL)
    {
        /* append to the asked list */
        xmlnode_insert_cdata(asked,"/",1);
        xmlnode_insert_cdata(asked,idstr,strlen(idstr));
        js_xdb_set(NULL,"jabber:http:asked",asked);

        /* ask them */
        pres = js_pres_new(JPACKET__SUBSCRIBE,jid_full(id),"Someone is aksing to check your presence from the web, would you like to allow this?  By doing so, you are allowing anyone from the web to see your presence at any time.");
        from = jid_new(rq->p,xmlnode_get_attrib(rq->x,"Host"));
        jid_set(from,"http/public",JID_RESOURCE);
        xmlnode_put_attrib(pres,"from",jid_full(from));
        js_deliver(jpacket_new(pres));
    }

    pres = ppdb_primary(svc_http_public_presence_cache,id);

    /* header */
    switch(ext[0])
    {
        case 'c':
        case 'g':
            /* simple redirect */
            if(pres != NULL)
                svc_http_redirect(rq,xmlnode_get_attrib(xmlnode_get_tag(x,"QS"),"on"));
            else
                svc_http_redirect(rq,xmlnode_get_attrib(xmlnode_get_tag(x,"QS"),"off"));
            return 1;
        case 'h':
            svc_http_reply_mime(rq,MIME_HTML);
            if(pres != NULL)
                svc_http_reply_append(rq,"<HEAD><TITLE>",user," is online</TITLE></HEAD><BODY><H1 ALIGN='center'>",user," is online</H1>",rq);
            else
                svc_http_reply_append(rq,"<HEAD><TITLE>",user," is offline</TITLE></HEAD><BODY><H1 ALIGN='center'>",user," is offline</H1>",rq);
            break;
        case 'j':
            svc_http_reply_mime(rq,MIME_JS);
            if(pres != NULL)
                svc_http_reply_append(rq,"<SCRIPT>jabber_user = '",user,"';jabber_user_available = true;",rq);
            else
                svc_http_reply_append(rq,"<SCRIPT>jabber_user = '",user,"';jabber_user_available = false;",rq);
            break;
        case 'x':
            svc_http_reply_mime(rq,MIME_XML);
            svc_http_reply_append(rq,"<jabber>",rq);
            break;
    }

    /* body */
    while((pres = ppdb_get(svc_http_public_presence_cache,id)) != NULL)
    {
        switch(ext[0])
        {
            case 'h':
                /* convert presence to HTML */
                res = xmlnode_get_attrib(pres,"from");
                res = strstr(res,"/");
                if(res != NULL)
                {
                    ++res;
                    if(xmlnode_get_attrib(pres,"type") == NULL)
                        svc_http_reply_append(rq,"<H2>",res,"</H2>",rq);
                    else
                        svc_http_reply_append(rq,"<BIG>",res,"</BIG>",rq);
                }

                svc_http_reply_append(rq,"<UL>",rq);
                for(cur = xmlnode_get_firstchild(pres); cur != NULL; cur = xmlnode_get_nextsibling(cur))
                    if(xmlnode_get_type(cur) == NTYPE_TAG && xmlnode_get_attrib(cur,"xmlns") == NULL)
                        svc_http_reply_append(rq,"<LI><B>",xmlnode_get_name(cur),"</B>: ",xmlnode_get_data(cur),"</LI>",rq);
                svc_http_reply_append(rq,"</UL>",rq);

                break;
            case 'j':
                /* convert presence to javascript */
                break;
            case 'x':
                svc_http_reply_append(rq,xmlnode2str(pres),rq);
                break;
        }
    }

    /* cleanup */
    switch(ext[0])
    {
        case 'h':
            svc_http_reply_append(rq,"</BODY></HTML>",rq);
            break;
        case 'j':
            svc_http_reply_append(rq,"</SCRIPT>",rq);
            break;
        case 'x':
            svc_http_reply_append(rq,"</jabber>",rq);
            break;
    }

    svc_http_reply(rq);
    return 1;
}

int svc_http_public_hrt(hrq rq, xmlnode x)
{
    if(x == NULL) return 0;

    return 1;
}

int svc_http_public_message(hrq rq, xmlnode x)
{
    xmlnode msg, cur;
    char *user, *idstr, *res;
    jid id, from;

    if(x == NULL) return 0;
    user = xmlnode_get_attrib(x,"FILE");
    if(user == NULL || strlen(user) == 0) return 0;

    /* the user we're curious about */
    id = jid_new(rq->p,xmlnode_get_attrib(rq->x,"Host"));
    jid_set(id,user,JID_USER);
    idstr = jid_full(id);
    idstr = strstr(idstr,":");
    ++idstr; /* just the user@host */

    /* create the message */
    msg = xmlnode_new_tag("message");
    xmlnode_put_attrib(msg,"to",jid_full(id));

    /* generate a decent from server/http/public/12.34.56.78 */
    from = jid_new(rq->p,xmlnode_get_attrib(rq->x,"Host"));
    res = malloc(13 + strlen(rq->ip));
    *res = '\0';
    strcat(res,"http/public/");
    strcat(res,rq->ip);
    jid_set(from,res,JID_RESOURCE);
    free(res);
    xmlnode_put_attrib(msg,"from",jid_full(from));

    /* fill it in with the query string */
    for(cur = xmlnode_get_firstattrib(xmlnode_get_firstchild(x)); cur != NULL; cur = xmlnode_get_nextsibling(cur))
        xmlnode_insert_cdata(xmlnode_insert_tag(msg,xmlnode_get_name(cur)),xmlnode_get_data(cur),xmlnode_get_datasz(cur));

    /* deliver it */
    log_debug(ZONE,"delivering message");
    js_deliver(jpacket_new(msg));

    /* reply */
    svc_http_reply_mime(rq,MIME_HTML);
    svc_http_reply_append(rq,"<H1>Message delivered to ",idstr,"</H1>",rq);
    svc_http_reply(rq);

    return 1;
}

/* branches out the request */
void svc_http_public(hrq rq)
{
    if(svc_http_public_presence(rq,xmlnode_get_tag(rq->x,"public/presence"))) return;
    if(svc_http_public_hrt(rq,xmlnode_get_tag(rq->x,"public/hrt"))) return;
    if(svc_http_public_message(rq,xmlnode_get_tag(rq->x,"public/message"))) return;
    svc_http_error(rq,404);
}

mreturn svc_http_public_mapi(mapi m, void *arg)
{
    if(m->packet->type == JPACKET_IQ) return M_IGNORE;

    if(m->packet->to->resource != NULL && strncmp(m->packet->to->resource,"http/public",11) != 0) return M_PASS;

    log_debug(ZONE,"checking packet");

    switch(m->packet->type)
    {
        case JPACKET_PRESENCE:
            js_delay(m->packet->x,"received");
            svc_http_public_presence_cache = ppdb_insert(svc_http_public_presence_cache,m->packet->from,m->packet->x);
            break;
        case JPACKET_MESSAGE:
            js_bounce(m->packet->x,(terror){405,"The server is unable to deliver messages to this address"});
            break;
        case JPACKET_S10N:
            js_bounce(m->packet->x,(terror){405,"This address can not be subscribed to"});
            break;
    }

    return M_HANDLED;
}

void svc_http_public_init(void)
{
    svc_http_public_presence_cache = NULL;
    js_mapi_register(P_SERVER, svc_http_public_mapi, NULL);
}
