#include "svc_irc.h"

/* IN: Jabber XML --> IRC */

/* send a message to the client */
void svc_irc_inmsg(ircdata d, jpacket p)
{
    char *body, *oldbody;
    int i;

    body = xmlnode_get_tag_data(p->x,"body");
    if(body == NULL)
        body = pstrdup(p->p," ");

    if(jpacket_subtype(p) == JPACKET__ERROR)
    {
        svc_irc_line(d, NULL, "NOTICE", xmlnode_get_tag_data(p->x,"error"), 6, d->nick, ":Your message [",body,"] to ",jid_full(p->from),"failed");
        return;
    }

    /* translate /me to CTCP ACTION */
    if(strncmp(body,"/me ",4) == 0)
    {
        oldbody = body + 4;
        body = pmalloc(p->p,strlen(body) + 10);
        body[0] = 1;
        body[1] = '\0';
        strcat(body,"ACTION ");
        strcat(body,oldbody);
        i = strlen(body);
        body[i] = 1;
        body[i+1] = '\0';
    }

    if(xmlnode_get_attrib(svc_irc_item(d,p->from),"subscription") != NULL && jpacket_subtype(p) != JPACKET__CHAT)
        svc_irc_line(d, svc_irc_nick(d,p->from), "PRIVMSG", body, 1, ROOM);
    else
        svc_irc_line(d, svc_irc_nick(d,p->from), "PRIVMSG", body, 1, d->nick);

}

/* send presence to the client */
void svc_irc_inpres(ircdata d, jpacket p)
{
    xmlnode old, item, new;
    jid from;

    item = svc_irc_item(d, p->from);
    if(item != NULL)
        from = jid_new(p->p, xmlnode_get_attrib(item,"jid"));
    else
        from = NULL;

    old = ppdb_primary(d->pres,from);
    d->pres = ppdb_insert(d->pres,p->from,p->x);
    new = ppdb_primary(d->pres,from);

    if(old == NULL && new != NULL)
        svc_irc_line(d, "presence", "MODE", NULL, 3, ROOM, "+o", xmlnode_get_attrib(item,"name"));
    if(old != NULL && new == NULL)
        svc_irc_line(d, "presence", "MODE", NULL, 3, ROOM, "-o", xmlnode_get_attrib(item,"name"));
}

/* send a subscription request to the client */
void svc_irc_ins10n(ircdata d, jpacket p)
{
    char *from;

    from = jid_full(p->from);
    from = strstr(from,":");
    ++from;
    svc_irc_line(d, "nickserv", "NOTICE", NULL, 7, d->nick, ":Presence", xmlnode_get_attrib(p->x,"type"), "request from", from, ":", xmlnode_get_tag_data(p->x,"status"));
}

/* manage the roster for the client */
void svc_irc_inroster(ircdata d, jpacket p)
{
    xmlnode cur, inew, iold;
    jid id;
    spool members;
    char *ids;

    if(jpacket_subtype(p) == JPACKET__RESULT)
    { /* got our roster */
        d->roster = xmlnode_dup_pool(d->t->p, p->iq);
        members = spool_new(p->p);
        spooler(members, "@", d->nick, " ", members);
        for(cur = xmlnode_get_firstchild(d->roster); cur != NULL; cur = xmlnode_get_nextsibling(cur))
        {
            if((ids = xmlnode_get_attrib(cur,"jid")) != NULL)
            {
                svc_irc_nick(d, jid_new(p->p,ids)); /* makes sure name="" has a unique nick */
                if(j_strcmp(xmlnode_get_attrib(cur,"subscription"),"both") == 0 || j_strcmp(xmlnode_get_attrib(cur,"subscription"),"to") == 0)
                    spooler(members, "+", xmlnode_get_attrib(cur,"name"), " ", members);
                else
                    spooler(members, xmlnode_get_attrib(cur,"name"), " ", members);
            }
        }
        svc_irc_line(d,NULL,"353",spool_print(members),3,d->nick,"=",ROOM);

        if(d->whoflag)
        {
            svc_irc_who(d, ROOM);
            d->whoflag = 0;
        }
        return;
    }

    /* handle roster pushes */
    if(jpacket_subtype(p) != JPACKET__SET)
        return;

    inew = xmlnode_get_firstchild(p->iq);
    id = jid_new(p->p,xmlnode_get_attrib(inew,"jid"));
    iold = svc_irc_item(d,id);
    xmlnode_hide(iold);

    xmlnode_insert_tag_node(d->roster,inew);
    svc_irc_nick(d,id); /* make sure it has a nick */

    if(xmlnode_get_attrib(iold,"subscription") == NULL)
        svc_irc_line(d,svc_irc_nick(d,id),"JOIN",NULL,1,ROOM);
    if(xmlnode_get_attrib(iold,"name") != NULL && xmlnode_get_attrib(inew,"name") != NULL && j_strcmp(xmlnode_get_attrib(iold,"name"),xmlnode_get_attrib(inew,"name")) != 0)
        svc_irc_line(d,svc_irc_nick(d,id),"NICK",NULL,1,xmlnode_get_attrib(inew,"name"));

}

/* incoming data from the session to the client */
void svc_irc_in(session s, jpacket p, void *arg)
{
    ircdata d = (ircdata)arg;

    if(p == NULL) return; /* XXX session ended, do */

    log_debug(ZONE,"sent %s",xmlnode2str(p->x));

    /* debug tab 
    svc_irc_line(d,"debug","PRIVMSG",xmlnode2str(p->x),1,d->nick);
    */

    /* send to client */
    switch(p->type)
    {
        case JPACKET_MESSAGE:
            svc_irc_inmsg(d,p);
            break;
        case JPACKET_PRESENCE:
            svc_irc_inpres(d,p);
            break;
        case JPACKET_S10N:
            svc_irc_ins10n(d,p);
            break;
        case JPACKET_IQ:
            if(NSCHECK(p->iq,NS_ROSTER))
                svc_irc_inroster(d,p);
            break;
    }

    xmlnode_free(p->x);
}
