#ifndef XDB_MYSQL_H_INCLUDED
#define XDB_MYSQL_H_INCLUDED

#include <mysql/mysql.h>

/*
 * Type definitions
 */

/* a single bound variable in a query definition */
typedef struct query_def_bind_var_struct
{
  struct query_def_bind_var_struct *next;  /* next variable binding */
  struct query_def_bind_var_struct *prev;  /* previous variable binding */
  const char *name;                        /* variable name */
  const char *replace_text;                /* text to replace in query */
  const char *default_value;               /* default value to replace in query */
  int need_escape;                         /* does it need escaping? */
} *query_def_bind_var;

/* Query definition */
typedef struct query_def_struct
{
  pool p;                        /* private memory pool for querydef */
  char *query_text;              /* the query text */
  query_def_bind_var first_var;  /* the first query variable */
  query_def_bind_var last_var;   /* the last query variable */
} *query_def;

/* Single entry in the column map */
typedef struct col_map_entry_struct
{
  struct col_map_entry_struct *next;  /* pointer to next column map entry */
  const char *name;                   /* column variable name */
  int index;                          /* index of column in row results */
} *col_map_entry;

/* Column map */
typedef struct col_map_struct
{
  pool p;                             /* memory pool for column map entries */
  col_map_entry first;                /* first column map entry */
  col_map_entry last;                 /* last column map entry */
} *col_map;

/*
 * Function prototypes
 */

/* xdb_mysql.h */
extern void xdb_mysql(void);
extern int xdbmysql_test_user(const char *user);

/* xdb_mysql_auth.c */
extern xmlnode xdbmysql_auth_get(const char *user);
extern int xdbmysql_auth_set(const char *user, xmlnode data);

/* xdb_mysql_config.c */
extern int is_true(const char *str);
extern int is_false(const char *str);
extern int xdbmysql_config_init(xmlnode cfgroot);
extern MYSQL *xdbmysql_database(void);
extern xmlnode xdbmysql_query_get(const char *name);
extern void xdbmysql_config_uninit(void);

/* xdb_mysql_offline.c */
extern xmlnode xdbmysql_offline_get(const char *user);
extern int xdbmysql_offline_set(const char *user, xmlnode data);

/* xdb_mysql_querydef.c */
extern query_def xdbmysql_querydef_init(xmlnode qd_root);
extern void xdbmysql_querydef_setvar(query_def qd, const char *name,
				     const char *value);
extern char *xdbmysql_querydef_finalize(query_def qd);
extern void xdbmysql_querydef_free(query_def qd);
extern col_map xdbmysql_colmap_init(xmlnode qd_root);
extern void xdbmysql_colmap_free(col_map cm);
extern int xdbmysql_colmap_index(col_map cm, const char *name);

/* xdb_mysql_register.c */
extern xmlnode xdbmysql_register_get(const char *user);
extern int xdbmysql_register_set(const char *user, xmlnode data);

/* xdb_mysql_roster.c */
extern xmlnode xdbmysql_roster_get(const char *user);
extern int xdbmysql_roster_set(const char *user, xmlnode roster);

#endif  /* XDB_MYSQL_H_INCLUDED */
